##########################################################################################################################################
# FSA project FS307037 
# Development of a modular assessment framework to quantify the risk of AMR exposure via food products: example of chicken and lettuce
##########################################################################################################################################
# File Name:     ui.R
# Description:   # Define UI for data upload app 

# Company:       Ausvet Eu
# Author:        Céline Faverjon
# Role:          Epidemiologist
# Email:         celine@ausvet.eu
# Date:          31.10.21
##########################################################################################################################################

ui <- fluidPage(
  
  # Sidebar layout  ----
  sidebarLayout( 
    
    # SIDE PANEL --------------------------------------------------------------------------------------------------------------------------
    sidebarPanel(width =12,
                 h1("AMR quantitative risk assessment", style="color:green"),
                 h5("Modular assessment framework to quantify the risk of antimicrobial resistance exposure via food products (Project FS307037).", style="color:green")
    ),
    
    # MAIN PANEL --------------------------------------------------------------------------------------------------------------------------
    mainPanel(width =12,
      
      tabsetPanel(
        
        # INTRODUCTION TAB --------------------------------------------------------------------------------------------------------------------------
        tabPanel("Introduction", 
                 h3("Introduction", style="color:green"),
                 
                 tags$hr(),
                 strong("About the project"),
                 p("This Shiny graphical user interface (GUI) has been developed for the Project FS307037."),
                 p("The aim of the project was to develop a stochastic and modular modelling framework to quantify consumers exposure to antimicrobial resistant (AMR) bacteria."),
                 p("The model has been developed and validated for two food products, chicken and lettuce and two microorganisms of public health interest E. coli and Campylobacter spp. However, the model can be adapted to different microorganisms and antimicrobial resistance genes in different value chains."), 
                 p("The purpose of this GUI is to allow the model users to easily use the model for other case studies."),
                 
                 tags$hr(),
                 strong("How to use this app?"),
                 p("This GUI allows users to select the production chain they are interested to customize their choice with specific parameters and explore the model outputs and the correlation analysis results. "),
                 p("The GUI is made of 5 sections that should be carefuly explored:"),
                 tags$div(tags$ul(
                   tags$li(tags$span("Introduction (where you are now): provide background information about the model.")),
                   tags$li(tags$span("Scenario: this section is used to define the model to be investigated and the estimated variables to be used.")),
                   tags$li(tags$span("Estimated variables: this section is used to perform simulation based on the data uploaded in the 'Scenario' tab.")),
                   tags$li(tags$span("Risk assessment: this section runs the model and provides the results of the analysis.")),
                   tags$li(tags$span("Correlation analysis: this section performs a global sensitivity analysis, or correlation analysis, to identify the most influential variables in the model."))
                 )),
                 p("Additional details are provided in each section."),
                 p("To use the model you should do the following:"),
                 img(src = "figShinyApp.png", height = 125, width = 750),
                 
                 tags$hr(),
                 strong("The modelling framework"),
                 p("The modelling framework is organized in 4 distinct modules that represent steps in the risk pathway:"),
                 tags$div(tags$ol(
                   tags$li(tags$span("Production module: include all the relevant on-farm practices having an influence on the probability of presence of bacteria carrying AMR genes in food.")),
                   tags$li(tags$span("Processing module: include all the food transformation processes from raw product to manufactured product including packaging and their associated probabilities of reducing or increasing bacteria load and AMR genes contamination in food.")),
                   tags$li(tags$span("Post-processing module: focus on transport and storage practices at retail having an influence on bacteria load and AMR genes contamination level")),
                   tags$li(tags$span("Home preparation module: include the key consumers behavior (e.g., washing lettuce or cooking meat) having an influence on the final AMR exposure which is a function of the prevalence and level of contamination of food units at the time of consumption."))
                   )),
                 p("The modules are linked together. It means that the output variables of the module 'Production' 
                   are used as input variables for the module 'Processing'. Similarly, the output variables of the module 'Processing' are used as input variables for the module 'Post-processing' etc.
                  However, for sake of simplicity, the data associated with each module are presented separately in this App."),
                 p("Each module is made of four different type of variables:"),
                 tags$div(tags$ul(
                   tags$li(tags$span("Selected variables = variables defined by the model user before running the analysis. They are used to define a particular model scenario, including the value chain and the hazard risk pathway considered in the risk analysis.")),
                   tags$li(tags$span("Estimated variables =  variables estimated by the model user based on the literature. They are often expressed as probability distributions.")),
                   tags$li(tags$span("Calculated variables = variables calculated based on the value of the selected and estimated variables previously defined. For example, the number of bacteria on a portion of chicken meat after X days spent in a fridge at Y °C was calculated based on the estimated variable 'minimum growth temperature'.")),
                   tags$li(tags$span("Output variables = special kind of calculated variables used to estimate the risk of AMR bacteria/gene exposure at the end of each module. They are the key variables used as results of the risk analysis. Their value is presented in terms of probability distribution, median and 95% prediction intervals. 
                   As key variables of interest, the output variables are also the target for the correlation analysis."))
                 )),
                 p("The stochastic model is built in R (R Development Core Team 2019) and uses several R packages to simulate probability distribution, compute and visualize the results."),

                 img(src = "chickenModel.png", height = 400, width = 600),
                 br(),
                 em("fig1. Overview of the model structure and key outputs for the chicken value chain. A contaminated serving is defined as a meal containing antimicrobial resistance bacteria. Red arrows = positive flocks, black arrows = negative flocks", style="color:grey"),
                 br(),
                 br(),
                 br()
                 ), 
        
        # INPUT data TAB --------------------------------------------------------------------------------------------------------------------------
        tabPanel("Scenario", 
                 fluidRow(
                   box(width=12, 
                       h3("Scenario", style="color:green"),
                       p("This tab allows you to define the scenario and case study you want to investigate. It also allows you to upload the Excel file with inputs parameters associated with the estimated variables and define the number of simulations you want to perform.")

                      
                   )
                 ),
                 
                 # Horizontal line ----
                 tags$hr(),
                 strong("1. Define your scenario:", style="color:darkgreen"),
                 p("Please customize the risk pathway, or scenario, you want to investigate."),
                 
                 fluidRow(
                   box(width=12, 
                       #Food product?
                       selectInput(inputId="Product", label="Select food production chain", choices=c("Chicken", "Lettuce"))
                   ),
                   
                   box(width=6, 
                       #Product sold as whole carcass or portion?
                       selectInput(inputId="Product_cut", label="Is the product cut?", choices=c("yes", "no")),
                       
                       #packaging type?
                       selectInput(inputId="Pack_type", label="What type of packaging is considered?", choices=c("MAP", "None")),
                       p("MAP = Modified Atmosphere Packaging")
                       
                   ),
                   
                   box(width=6, 
                       #selected variables specific to the selected food production chain
                       uiOutput("productSelection")
                       
                   )
                   
                 ),
                 
                 # Horizontal line ----
                 tags$hr(),
                 strong("2. Define your case study:", style="color:darkgreen"),
                 p("Please define the hazard attributes."),
                 
                 fluidRow(
                   box(width=6, 
                       #Bacteria?
                       textInput(inputId="Bacteria", label="Microorganism", value="E.coli"),

                   ),
                   box(width=6, 
                       #resistance gene?
                       textInput(inputId="Gene", label="Resistance gene", value="CMY2")
                   )
                 ),
                 
                 # Horizontal line ----
                 tags$hr(),
                 strong("3. Define modules to be included in the analysis:", style="color:darkgreen"),
                 p("Add explanation on how to select the modules"),
                 fluidRow(
                   box(width=12,
                     #Modules to be included in the analysis
                     selectInput(inputId="Modules", label="Modules to be included", choices=c("Production > Processing > Post-processing > Home preparation", 
                                                                                                     "Processing > Post-processing > Home-preparation", 
                                                                                                     "Post-processing > Home-preparation", 
                                                                                                     "Home-preparation")
                                 ),
                   ),
                   box(width=6,
                     #selected variables specific to the selected food production chain
                     uiOutput("moduleSelection_prev"),
                     uiOutput("distribSelection_prev")
                   )
                   ,
                   box(width=6,
                       #selected variables specific to the selected food production chain
                       uiOutput("moduleSelection_cfu"),
                       uiOutput("distribSelection_cfu")
                   )
                 ),
                 
                 # Horizontal line ----
                 tags$hr(),
                 strong("4. Define parameters used of the analysis:", style="color:darkgreen"),
                 p("Please upload the Excel file with inputs parameters associated with the relevant estimated variables, and define the number of simulations to be performed."),
                 
                 fluidRow(
                   box(width=6, 
                       # Input: Select a file ----

                       fileInput("estimatedVariables", "Choose EXCEL File",
                                 multiple = FALSE,
                                 accept = c(".xls",
                                            ".xlsx")
                       ),
                       em("NB: Be very careful to upload the right Excel spreadsheet since this represents the data input for the model.")
                   ),
                   box(width=6, 
                       #select number of simulations to be performed
                       numericInput(inputId="Runs", label="Number of simulations to be performed", value=100000),
                       em("NB: The number of Monte Carlo simulations performed can be customized as preferred (however we recommend to run the models with > 10.000 simulation). ")
                    
                   )
                 ),
                 # Horizontal line ----
                 tags$hr(),
                 strong("5. Next step", style="color:darkgreen"),
                 p("Once you have completed the steps 1 to 4, go to the tab 'Estimated variables'."),
                 br(),
                 br()
                
        ), 
        
        # SIMULATION TAB --------------------------------------------------------------------------------------------------------------------------
        tabPanel("Estimated variables", 
                 fluidRow(
                   box(width=12, 
                       h3("Estimated variables", style="color:green"),
                       p("This tab allows you to simulate multiple values for the 'estimated variables' used in each module based on Monte Carlo simulations, and on the probability distributions defined in the excel spreadsheet uploaded in the 'Scenario' tab."),
                       p("Briefly, Monte Carlo simulations select random values from each estimated variable distribution and provide outputs as distributions that indicate the likelihood of each result. "),
                       
                       #action button to perform simulation
                       tags$hr(),
                       strong("1.	Chick on the 'Run simulations' button to perform the simulations."),
                       p("Every time you click the simulation button, a new set of data is simulated and results generated will be different."),
                       actionButton("buttonSimulation", "Run simulations", class = "btn btn-success"),
                       
                       tags$hr(),
                       strong("2.	Select the module for which you would like to look at the variables distribution." ),
                       #selected module to be investigated in detail
                       uiOutput("moduleSelection_Simulation"),
                       
                       tags$hr(),
                       strong("3.	Results of the simulations displayed per module"),
                       p("Graphs generated can be downloaded and saved as png."),
                       p("NB: Check the results of the simulation carefully to make sure you uploaded the right data!" , style="color:green"),
                       #table with estimated variables description
                       withSpinner(tableOutput("simulatedVariablesDescription")),
                       
                       #plot of the estimated variables simulated
                       withSpinner(plotOutput("plotSimulation", height=1000)),
                       uiOutput("get_plotSimulation"),
                       br(),
                       br()
                   )
                 )
   
        ), 
        
        # RISK ASSESSMENT TAB --------------------------------------------------------------------------------------------------------------------------
        tabPanel("Risk assessment", 
                 fluidRow(
                   box(width=12, 
                       h3("Results risk assessment", style="color:green"),
                       p("For each module (production, processing, post-processing, home preparation) this tab provides an overview of the values of the output variables (concentration and prevalence). The calculations are based on the values provided in the uploaded excel spreadsheet, the selected number of simulations and the 'Estimated variables' displayed in the previous tab. "),
                       p("Because we are using a stochastic model, all the outcome variables are probabilities distributions."),
                       
                       #action button to perform simulation
                       tags$hr(),
                       strong("1.	Click on the 'Run calculations' button to perform the calculation"),
                       br(),
                       actionButton("buttonRiskAssessment", "Run calculations", class = "btn btn-success"),
                                              
                       # Download raw data ----
                       tags$hr(),
                       strong("2. Download the results"),
                       p("The full results associated with the risk analysis, including value of the estimated and calculated variable of each simulation performed, are available for download below."),
                       uiOutput("get_rawData"),
                       
                       #Results ---
                       tags$hr(),
                       strong("3. Visualize the results"),
                       br(),
                       htmlOutput("foodItem_description"),
                       br(),
                       p("Graphs generated can be downloaded and saved as png."),
                       #table overall results
                       #centile to be included in the analysis
                       sliderInput(inputId="centile", label="Centiles to be calculated:", value=c(0.05, 0.975), min =0, max=1
                       ),
                       br(),
                       withSpinner(tableOutput("overallRisk")),
                       uiOutput("get_overallRisk"),
                       br(),
                       withSpinner(tableOutput("specificRisk")),
                       uiOutput("get_specificRisk"),
                       br(),
                       p("*Adjusted values mean that the birds and carcasses with very low contamination level (< 1CFU/carcass) have been considered as non contaminated for the calculations."),
                                              
                       #selected module to be investigated in detail
                       br(),
                       uiOutput("moduleSelection_Risk"),
                       #centile to be included in the analysis
                       sliderInput(inputId="centile2", label="Range of centiles to be visulalized in the figures below:", value=c(0,1), min =0, max=1
                       ),
                       #plot probability distribution, outcome selected module
                       withSpinner(plotOutput("outcomeDistribution")),
                       uiOutput("get_plotRisk1"),
                       
                       br(),
                       
                       #plot probability distribution, outcome selected module per flock of origin
                       withSpinner(plotOutput("outcomeDistributionFlock")),
                       uiOutput("get_plotRisk2")
                   )
                 ),
                 br(),
                 br()
        ), 
        
        # correlation ANALYSIS TAB --------------------------------------------------------------------------------------------------------------------------
        tabPanel("Correlation analysis",
                 fluidRow(
                   box(width=12, 
                       h3("Correlation analysis", style="color:green"),
                       p("This tab allows you to perform a global sensitivity, or correlation, analysis using the simulated and calculated data generated in the three previous tabs. The objective of the sensitivity analysis is to evaluate the impact of variability and uncertainty in the estimated variables on the output variables."),
                       p("The correlation between the values of the estimated variables and the outcome variables is calculated using Spearman’s rank correlation coefficients."),
                       
                       #action button to correlation analysis
                       tags$hr(), 
                       strong("1.	Click on the 'Run correlation analysis' button to perform a global sensitivity analysis" ),
                       br(),
                       actionButton("buttoncorrelation", "Run correlation analysis", class = "btn btn-success"),
                       
                       # Download SA data ----
                       tags$hr(), 
                       strong("2. Download the results"),
                       p("The full data associated with the outputs of the correlation analysis are available for download below."),
                       uiOutput("get_saData"),
                       
                       # SA plot
                       tags$hr(),
                       strong("3.  Visualize the results per module"),
                       p("The correlation between the values of the estimated variables and the outcome variables is calculated using Spearman rank correlation coefficients. Only estimated variables with spearman correlation coefficient above |0.025| have been included in the figures. The estimated variables with a lower Spearman correlation coefficient were considered has having a negligible effect on the model outcomes and are not shown. "),
                       p("Graphs generated can be downloaded and saved as png."),
                       #selected module to be investigated in detail
                       uiOutput("moduleSelection_SA"),
                       #plot tornado chart of the results of the SA associated with the outcome of the selected module
                       withSpinner(plotOutput("outcomeSA")),
                       uiOutput("get_plotRisk3")

                   )
                 ),
                 br(),
                 br()

        )
        
      )
      
    )
    
  )
)

