#########################################################################################################################################
# FSA project FS307037 
# Development of a modular assessment framework to quantify the risk of AMR exposure via food products: example of chicken and lettuce
##########################################################################################################################################
# File Name:     Function_DataSimulation.R
# Description:   Set of functions to simulate a dataset of N row and Y columns based on probabilities distribution

# Company:       Ausvet Eu
# Author:        Céline Faverjon
# Role:          Epidemiologist
# Email:         celine@ausvet.eu
# Date:          03.02.21
##########################################################################################################################################

#packages of interest
packages = c("mc2d", "MCSim", "stats", "EnvStats", "extraDistr", "remotes", "FAdist")

## load or install&load all
package.check <- lapply(
  packages,
  FUN = function(x) {
    if (!require(x, character.only = TRUE)) {
      install.packages(x, dependencies = TRUE)
      library(x, character.only = TRUE)
    }
  }
)
source("www/Function_ProbaDistribution.R")



g_legend<-function(a.gplot){
  #this function allowed to easily extract a legend from a ggplot
  tmp <- ggplot_gtable(ggplot_build(a.gplot))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  legend <- tmp$grobs[[leg]]
  return(legend)
}

dataSimulation <- function(data, input, N){
  # this function aims at simulating data based on an input dataframe provided by the user
  
  # outputData = dataframe to be merged with inputData
  
  # inputData = dataframe defining a list of variables to be simulated (inputData$variable), 
  # the probability distribution associated with each variable (inputData$Distribution), truncated or not (inputData$Truncated),
  # and the parameters that must be used as input of the probability distrinution (min, max, mean, SD, Other, Other2)
  
  #N = number of simulation to be performed for each variable
  
  for(i in 1:nrow(input)){
    
    if(is.na(input$Distribution[i]) | input$Distribution[i] =="None" ){
      data$variable <-0
      
    }else if (input$Distribution[i] == "Point estimate"){
      data$variable <- input$Other[i]
      
    }else if (input$Distribution[i]== "Uniform"){
      data$variable <- runif(N,input$Min[i],input$Max[i])
      
    }else if (input$Distribution[i]=="Triangular"){
      data$variable <- mc2d::rtriang(N, min=input$Min[i], mod=input$Mean_Mode[i], max=input$Max[i])
      
    }else if (input$Distribution[i]== "Pert"){
      data$variable <- rpert(N, input$Min[i], input$Mean_Mode[i], input$Max[i])
      
    }else if (input$Distribution[i] =="Normal"){
      if(input$Truncated[i]=="Yes"){
        data$variable <- rnormt(N, min=input$Min[i], max=input$Max[i], input$Mean_Mode[i], input$SD[i])
      }else {
        data$variable <- rnorm(N, input$Mean_Mode[i], input$SD[i])
      }
      
    }else if (input$Distribution[i]== "Beta"){
      if(input$Truncated[i]=="Yes"){
        data$variable <- rbetat(N,min=input$Min[i], max=input$Max[i],shape1=input$Other[i],shape2=input$Other2[i])
      }else {
        data$variable <- rbeta(N,shape1=input$Other[i],shape2=input$Other2[i])
      }
      
    }else if (input$Distribution[i]== "Poisson"){
      if(input$Truncated[i]=="Yes"){
        data$variable <- rpoist(N, min=input$Min[i], max=input$Max[i], input$Other[i])
      }else {
        data$variable <- rpois(N, input$Other[i])
      }
      
    }else if (input$Distribution[i]== "Negbin"){
      data$variable <- rnegbin(N, mu=input$Mean_Mode[i], theta=input$Other[i])
      
    }else if (input$Distribution[i]== "Binomial"){
      data$variable <- rbinom(N, 1, prob=input$Other[i])
      
    }else if (input$Distribution[i] =="Laplace"){
      if(input$Truncated[i]=="Yes"){
        data$variable <- rtrunclaplace(N, mean=input$Mean_Mode[i], scale=input$Other[i], lower=input$Min[i], upper=input$Max[i])
      }else {
        data$variable <- rlaplace(N, mu=input$Mean_Mode[i], sigma=input$Other[i])
      }
      
    }else if (input$Distribution[i] =="LogNormal" ){
      if(input$Truncated[i] =="Yes"){
        data$variable <- rlnormTrunc(N, meanlog=input$Mean_Mode[i], sdlog=input$SD[i], min=input$Min[i], max=input$Max[i])
      }else {
        data$variable <- rlnorm(N, meanlog=input$Mean_Mode[i], sdlog=input$SD[i])
      }
      
    }else if (input$Distribution[i] =="LogLogistic" ){
      if(input$Truncated[i] =="Yes"){
        data$variable <- rllogt(N, shape=input$Other[i], scale=input$Other2[i], min=input$Min[i], max=input$Max[i])
      }else {
        data$variable <- rllog(N, shape=input$Other[i], scale=input$Other2[i])
      }
      
    }else if (input$Distribution[i] =="Shifted LogLogistic" ){
      if(input$Truncated[i] =="Yes"){
        data$variable <-  rllog3t(N, shape=input$Other[i], scale=input$Other2[i], thres=input$Other3[i], min=input$Min[i], max=input$Max[i])
      }else {
        data$variable <- rllog3(N, shape=input$Other[i], scale=input$Other2[i], thres=input$Other3[i])
      }
      
    }else if (input$Distribution[i] =="Gamma"){
      if(input$Truncated[i] =="Yes"){
        data$variable <- rgammat(N, min=input$Min[i], max=input$Max[i], shape=input$Other[i], scale=input$Other2[i])
      }else {
        data$variable <- rgamma(N, shape=input$Other[i], scale=input$Other2[i])
      }
      
    }
    
    colnames(data)[ncol(data)]<- input$Variable[i] 
    
  }
  
  data
}

